#include "Engine.h"
#include "Room.h"
#include "Meeting.h"
#include "ConnectionSettings.h"
#include "Configuration.h"
#include "DisplaySettings.h"
#include "CommunicationManager.h"
#include "DeviceManager.h"
#include "Clock.h"
#include "ErrorMapper.h"
#include "WeeklyViewWidget.h"

#include <QApplication>
#include <QTimer>
#include <QList>
#include <QtDebug>

QTime Engine::endOfTheDay = QTime( 23, 59, 0, 0 ); // end of the day is 11:59pm
const int IDLE_TIME_MULTIPLIER = 60000; // Multiplies milliseconds to minutes

Engine::Engine() :
		iClock( 0 ), iConfiguration( Configuration::instance() ), iCommunication( 0 )
{
	qDebug() << "Engine::Engine()";
	// if reading of configuration fails, signal that initialization failed
	if ( iConfiguration == 0 )
	{
		QTimer::singleShot( 0, this, SLOT( closeApplication() ) );
		return;
	}
		
	//initialize window manager
	iWindowManager = new WindowManager( iConfiguration );
	connect( iWindowManager, SIGNAL( roomStatusInfoNeeded( Room * ) ), this, SLOT( roomStatusInfoNeeded( Room * ) ) );
	connect( iWindowManager, SIGNAL( observedEventDetected() ), this, SLOT( observedEventDetected() ) );
	connect( iWindowManager, SIGNAL( meetingActivated( Meeting * ) ), this, SLOT( fetchMeetingDetails( Meeting * ) ) );
	connect( iWindowManager, SIGNAL( currentRoomChanged( Room * ) ), this, SLOT( currentRoomChanged( Room * ) ) );
	connect( iWindowManager, SIGNAL( shownWeekChanged( QDate ) ), this, SLOT( shownWeekChanged( QDate ) ) );
	connect( iWindowManager, SIGNAL( passwordEntered( PasswordDialog::PasswordStatus ) ),
			this, SLOT( passwordEntered( PasswordDialog::PasswordStatus ) ) );

	
	// initialize communication
	iCommunication = new CommunicationManager( *(iConfiguration->connectionSettings()) );
	connect( iCommunication, SIGNAL( error( int, CommunicationManager::CommunicationType  ) ),
			this, SLOT( errorHandler( int ) ) );
	connect( iCommunication, SIGNAL( meetingsFetched( const QList<Meeting*>& ) ),
			this, SLOT( meetingsFetched( const QList<Meeting*>& ) ) );
	connect( iCommunication, SIGNAL( meetingDetailsFetched( Meeting& ) ),
			this, SLOT( meetingDetailsFetched( Meeting& ) ) );

	//initialize idle time counter
	iIdleTimeCounter = new QTimer();
	iIdleTimeCounter->setSingleShot( true );
	iIdleTimeCounter->setInterval( IDLE_TIME_MULTIPLIER * iConfiguration->displaySettings()->screensaver() );
	iIdleTimeCounter->start();
	connect( iIdleTimeCounter, SIGNAL( timeout() ), iWindowManager, SLOT( showRoomStatus() ) );

	// create application clock
	iClock = new Clock;
	connect( iClock, SIGNAL( tick( QDateTime ) ), this, SLOT( checkStatusOfAllRooms() ) );
	connect( iClock, SIGNAL( tick( QDateTime ) ), iWindowManager, SLOT( distributeDateTimeInfo( QDateTime ) ) );

	iAutoRefresh = new QTimer;
	iAutoRefresh->setInterval( iConfiguration->connectionSettings()->refreshInterval() * 1000 );
	iAutoRefresh->start();
	connect( iAutoRefresh, SIGNAL( timeout() ), iAutoRefresh, SLOT( start() ) );
	connect( iAutoRefresh, SIGNAL( timeout() ), this, SLOT( fetchMeetings() ) );

	// create device manager
	iDevice = new DeviceManager( iConfiguration->startupSettings() );
	connect( iDevice, SIGNAL( error( int, const QString& ) ), this, SLOT( errorHandler( int, const QString& ) ) );	
	connect( iDevice, SIGNAL( changeModeOrdered( DeviceManager::OperationMode ) ), 
			this, SLOT( changeModeOrdered( DeviceManager::OperationMode ) ) );
	iDevice->initDeviceManager();
	
	if( iDevice->currentOperationMode() == DeviceManager::KioskMode )
		iWindowManager->fullScreen();

	QTimer::singleShot( 0, this, SLOT( fetchMeetings() ) );

	// TODO: continue implementation
}

Engine::~Engine()
{
	qDebug() << "Engine::~Engine()";
	while ( !iMeetings.isEmpty() )
		delete iMeetings.takeFirst();
	iIdleTimeCounter->stop();
	delete iIdleTimeCounter;
	iIdleTimeCounter = 0;
	delete iWindowManager;
	iWindowManager = 0;
	delete iClock;
	iClock = 0;
	delete iDevice;
	iDevice = 0;
}

void Engine::closeApplication()
{
	qDebug() << "Engine::closeApplication()";
	// closes application after 1 second
	QTimer::singleShot( 1000, QApplication::instance(), SLOT( quit() ) );
}

void Engine::observedEventDetected()
{
	qDebug() << "Engine::observedEventDetected()";
	iWindowManager->showWeeklyView();
	// prepare to restart idle counter
	if ( iIdleTimeCounter->isActive() )
	{
		iIdleTimeCounter->stop();
	}
	// (re)start idle counter
	iIdleTimeCounter->start();
}

Room* Engine::defaultRoom()
{
	qDebug() << "Engine::defaultRoom()";
	return iConfiguration->defaultRoom();
}

void Engine::checkStatusOfAllRooms()
{
	qDebug() << "Engine::checkStatusOfAllRooms()";
	// iterate trough on the rooms
	for ( int i = 0; i < iConfiguration->rooms().count(); i++ )
	{
		// and check the status
		roomStatusInfoNeeded( iConfiguration->rooms().at( i ) );
	}
}

int Engine::indexOfMeetingAt( Room *aRoom, QDateTime aAt )
{
	qDebug() << "Engine::indexOfMeetingAt( Room *, QDateTime )";
	for ( int i = 0; i < iMeetings.count(); i++ )
	{
		// exchange server ensures that there is only one meeting in a room at a specified time
		if ( aRoom->equals( iMeetings.at( i )->room() )
			  && iMeetings.at( i )->startsAt() <= aAt
			  && iMeetings.at( i )->endsAt() >= aAt )
		{
			return i;
		}
	}
	return -1;
}

int Engine::indexOfMeetingAfter( Room *aRoom, QDateTime aAfter )
{
	qDebug() << "Engine::indexOfMeetingAfter( Room *, QDateTime )";
	// seeks for the next meeting on the SAME DAY
	int min = -1;
	for ( int i = 0; i < iMeetings.count(); i++ )
	{
		// if the meeting is in the same room, on the same day but after the specified time
		if ( aRoom->equals( iMeetings.at( i )->room() )
			  && iMeetings.at( i )->startsAt().date() == aAfter.date()
			  && iMeetings.at( i )->startsAt() > aAfter )
		{
			// if there was not any meeting find yet or the previously found is a later one then the (i)th
			if ( min == -1
				  || iMeetings.at( min )->startsAt() > iMeetings.at( i )->startsAt() )
			{
				min = i;
			}
		}
	}
	return min;
}

void Engine::roomStatusInfoNeeded( Room *aRoom )
{
	qDebug() << "Engine::roomStatusInfoNeeded( Room * )";
	if ( aRoom == 0 )
	{
		return;
	}

	int indexOfCurrentMeeting = indexOfMeetingAt( aRoom, iClock->datetime() );
	int indexOfNextMeeting = indexOfMeetingAfter( aRoom, iClock->datetime() );

	// if there is no meeting, then status is Free; otherwise Busy
	Room::Status status = ( indexOfCurrentMeeting == -1 ) ? Room::FreeStatus : Room::BusyStatus;
	// if room is Busy, then check end time, otherwise...
	QTime until = ( status == Room::BusyStatus ) ? iMeetings.at( indexOfCurrentMeeting )->endsAt().time() :
			  // ...if there is meeting following on the same day then check end time, otherwise end is the of the working day
			  (( indexOfNextMeeting != -1 ) ? iMeetings.at( indexOfNextMeeting )->startsAt().time() : Engine::endOfTheDay );

	//currently works only for deafult room
	if( aRoom->equals( *(defaultRoom() ) ) )
		iWindowManager->roomStatusChanged( aRoom, status, until );
}

void Engine::fetchMeetings()
{
	Room *room = defaultRoom();
	qDebug() << "Engine::fetchMeetings for " << room->name();
	fetchMeetings( iClock->datetime(), iClock->datetime().addDays( 7 ), room );
}

void Engine::fetchMeetingDetails( Meeting *aMeeting )
{
	qDebug() << "Engine::fetchMeetingDetails( Meeting* )";
	iWindowManager->showProgressBar( tr("Please Wait"), true );
	iWindowManager->updateProgressBar( tr("Fetching Meeting Details...") );
	connect( iWindowManager,
			 SIGNAL( progressBarCancelled() ),
			 this,
			 SLOT( fetchMeetingDetailsCancelled() )
			);
	iCommunication->fetchMeetingDetails( *aMeeting );
}

bool Engine::isMeetingInList( const QList<Meeting*> &aList, const Meeting *aMeeting )
{
	qDebug() << "Engine::isMeetingInList( const QList<Meeting*> &, const Meeting * )";
	for ( int i = 0; i < aList.count(); i++ )
	{
		if ( aMeeting->equals( *(aList.at( i )) ) )
		{
			return true;
		}
	}
	return false;
}

void Engine::meetingsFetched( const QList<Meeting*> &aMeetings )
{
	qDebug() << "Engine::meetingsFetched( const QList<Meeting*> & )";
	// check if there is any new meeting in the list came from the server -> added
	for ( int i = 0; i < aMeetings.count(); i++ )
	{
		// if the (i)th meeting is not in the local meeting list
		if ( !isMeetingInList( iMeetings, aMeetings.at( i ) ) )
		{
			// add to the local database =)
			Meeting* m = new Meeting( *(aMeetings.at( i )) );
			iMeetings.append( m );
			// and signal the changes
			iWindowManager->insertMeeting( m );
		}
	}

	// check if there is any meeting NOT in the list came from the server -> deleted
	for ( int i = 0; i < iMeetings.count(); i++ )
	{
		// if the (i)th meeting is in the local but NOT in the server's meeting list
		if ( !isMeetingInList( aMeetings, iMeetings.at( i ) ) )
		{
			Meeting* m = iMeetings.takeAt( i );
			// signal the changes
			iWindowManager->deleteMeeting( m );
			// delete the meeting from the local list
			delete m;
		}
	}

	// refresh room status info
	roomStatusInfoNeeded( defaultRoom() );
}

void Engine::meetingDetailsFetched( Meeting &aDetailedMeeting )
{
	qDebug() << "Engine::meetingDetailsFetched( Meeting & )";
	iWindowManager->closeProgressBar();
	iWindowManager->showMeetingInfo( &aDetailedMeeting );
}

void Engine::errorHandler( int aCode, const QString &aAddInfo )
{
	qDebug() << "Engine::ErrorHandler, aCode: " << aCode;
	// inform UI about the problem
	if( aCode >= 100 && aCode <= 150 ) { //communication errors
		//we don't want these to close operation changing
		qDebug() << "CommunicationManager signaled an error:" << aCode;
		iWindowManager->closeProgressBar();
	}
	iWindowManager->error( ErrorMapper::codeToString( aCode, aAddInfo ) );
}

void Engine::currentRoomChanged( Room *aCurrentRoom )
{
	qDebug() << "Engine::currentRoomChanged to " << aCurrentRoom->name();
	QDateTime from( iWindowManager->weeklyView()->beginnigOfShownWeek() );
	QDateTime to( from.addDays( 8 ) );
	fetchMeetings( from, to, aCurrentRoom );
}

void Engine::fetchMeetings( const QDateTime &aFrom, const QDateTime &aUntil, const Room *aIn )
{
	qDebug() << "Engine::fetchMeetings( const QDateTime &, const QDateTime &, const Room * )";
	iCommunication->fetchMeetings( aFrom, aUntil, *aIn );
}

void Engine::shownWeekChanged( QDate aFrom )
{
	qDebug() << "Engine::shownWeekChanged( QDate )";
	QDateTime from( aFrom );
	QDateTime to( aFrom.addDays( 7 ), QTime( 23, 59 ) );
	qDebug() << "Engine::shownWeekChanged " << aFrom.toString( "d.m. h:mm" ) << " to " << to.toString( "d.m. h:mm" );
	fetchMeetings( from, to, iWindowManager->weeklyView()->currentRoom() );
}

void Engine::changeModeOrdered( DeviceManager::OperationMode aMode )
{	
	qDebug() << "Engine::changeModeOrdered( DeviceManager::OperationMode )";
	QString message = tr( "You are about to change operation mode to %1." )
				.arg( iDevice->operationModeToString( aMode ) );
	
	iWindowManager->showPasswordDialog( iConfiguration->adminPassword(), message );
}

void Engine::passwordEntered( PasswordDialog::PasswordStatus aPasswordStatus )
{
	qDebug() << "Engine::passwordEntered( PasswordDialog::PasswordStatus )";
	iWindowManager->closePasswordDialog();
	
	switch ( aPasswordStatus )
	{
		case PasswordDialog::Correct :
		{
			iAutoRefresh->stop(); //we stop the metting updating
			iWindowManager->showProgressBar( "Changing current operation mode." );
			connect( iDevice, SIGNAL( changingMode( const QString & ) ),
					iWindowManager, SLOT( updateProgressBar( const QString & ) ) );
			connect( iDevice, SIGNAL( changeModeFailed() ), this, SLOT( changeModeFailed() ) );
			iDevice->changeMode( true );
			break;
		}
		case PasswordDialog::Incorrect :
		{
			iWindowManager->error( tr( "Incorrect password." ) );
			iDevice->changeMode( false );
			break;
		}
		default : //case PasswordDialog::Canceled
		{
			iDevice->changeMode( false );
		}
	}
}

void Engine::changeModeFailed()
{
	qDebug() << "Engine::changeModeFailed()";
	iWindowManager->closeProgressBar();
	iAutoRefresh->start(); //we start the metting updating
}

void Engine::fetchMeetingDetailsCancelled()
{
	iCommunication->cancelFetchMeetingDetails();
	iWindowManager->closeProgressBar();
}
