#ifndef ENGINE_H_
#define ENGINE_H_

#include <QObject>
#include <QDateTime>
#include "Room.h"
#include "WindowManager.h"

class QTimer;
class Clock;
class Configuration;
class CommunicationManager;
class Meeting;
class DeviceManager;

//! BusinessLogic class. Contains all the business logic of the application.
/*!
 * BusinessLogic class. Contains all the business logic of the application. It is responsible
 * for connecting user interface to lower application layers (IO).
 */
class Engine : public QObject
{
	Q_OBJECT

public:
	//! Constructor.
	/*!
	 * Constructor to initialize an Engine instance.
	 */
	Engine();
	//! Destructor.
	virtual ~Engine();
	//! Gets default room of the application.
	/*!
	 * Gets default room of the application.
	 * \return Pointer to the default Room instance.
	 */
	Room* defaultRoom();

signals:

	void meetingDetailsFetched( Meeting *aDetailedMeeting );	

private slots:
	//! Slot. Closes the application.
	/*!
	 * Slot. Closes the application.
	 */
	void closeApplication();
	//! Slot. Checks actual availability information of the specified room.
	/*!
	 * Slot. Checks actual availability information of the specified room.
	 * \param aRoom The room which availability information is needed.
	 */
	void roomStatusInfoNeeded( Room *aRoom );
	//! Slot. Indicates that some user event has happened.
	/*!
	 * Slot. Indicates that some user event has happened.
	 */
	void observedEventDetected();
	//! Slot. Asks the communication to fetch new meeting data.
	/*!
	 * Slot. Asks the communication to fetch new meeting data.
	 * \param aCurrentRoom The current room.
	 */
	void currentRoomChanged( Room *aCurrentRoom );
	//! Slot. Asks the communication to fetch new meeting data.
	/*!
	 * Slot. Asks the communication to fetch new meeting data.
	 * \param aCurrentRoom The current room.
	 */
	void shownWeekChanged( QDate aDate );
	//! Slot. Handles errors.
	/*!
	 * Slot. Handles errors and informs the UI by emitting the error() signal with the message in
	 * parameter.
	 * \param aCode The error code.
	 * \param aAddInfo Possible addition info.
	 */
	void errorHandler( int aCode, const QString &aAddInfo = "" );
	//! Slot. Fetches meetings from the server.
	/*!
	 * Slot. Fetches meetings from the server. Parameters are hard coded: the meetings of the default
	 * room from current and +/- 2 weeks are fetched.
	 */
	void fetchMeetings();
	//! Slot. Saves fetched meetings to the current instance's local storage.
	/*!
	 * Slot. Saves fetched meetings to the current instance's local storage. Meetings are soted in a
	 * private QList, it is iterated through and signals are emitted if there is any new, updated or
	 * deleted meeting.
	 * \param aMeetings The list of freshly fetched meetings.
	 */
	void meetingsFetched( const QList<Meeting*>& );
	
	void meetingDetailsFetched( Meeting &aDetailedMeeting );
	
	//! Slot. Checks the availability of all the rooms.
	/*!
	 * Slot. Checks the availability of all the rooms by iterating through the current object's local
	 * room storage and calling the roomStatusInfoNeeded() separately on each of them.
	 */
	void checkStatusOfAllRooms();
	//! Slot for popping up the confirmation dialog to change the current operation mode
	/*!
	 * Slot. Asks Window manager to pop up a confirmation dialog.
	 * \param aMode The operation mode to be changed to
	 */
	void changeModeOrdered( DeviceManager::OperationMode aMode );
	//! Slot. Fetches meeting details from the server.
	/*!
	 * Slot. Fetches meeting details from the server.
	 * \param aMeeting The meeting.
	 */
	void fetchMeetingDetails( Meeting *aMeeting );
	//! Slot for receiving the status of the entered password
	/*!
	 * Slot. Receives the status of the entered password and makes the DeviceManager to change the
	 * operation mode if the password is correct.
	 * \param aPasswordStatus The status of the password.
	 */
	void passwordEntered( PasswordDialog::PasswordStatus aPasswordStatus );
	//! Slot for receiving the failure event of operation mode changing.
	/*!
	 * Slot. Receives the failure event of operation mode changing.
	 */
	void changeModeFailed();
	//! Slot for receiving the cancel event of the progress bar.
	/*!
	 *  Receives the cancel event of the progress bar when meeting details requested.
	 */
	void fetchMeetingDetailsCancelled();
	
private:
	//! Provides the index of the Meeting instance which is at the specified time.
	/*!
	 * Provides the index of the Meeting instance which is at the specified time. If there are
	 * overlapping meetings then returns one of then undetetministically.
	 * \param aRoom The room which meetings are looked through.
	 * \param aAt Date and time when the meeting is already going.
	 * \return Index of the meeting if found; otherwise, -1.
	 */
	int indexOfMeetingAt( Room *aRoom, QDateTime aAt );
	//! Provides the index of the Meeting instance which is starts the closest to the specified time.
	/*!
	 * Provides the index of the Meeting instance which is starts the closest to the specified time.
	 * If there are overlapping meetings then returns one of then undetetministically.
	 * \param aRoom The room which meetings are looked through.
	 * \param aAt Date and time when the meeting is not yet started.
	 * \return Index of the meeting if found; otherwise, -1.
	 */
	int indexOfMeetingAfter( Room *aRoom, QDateTime aAfter );
	//! Indicates if the QList contains the Meeting or not.
	/*!
	 * Indicates if the QList contains the Meeting or not.
	 * \param aList List of meetings.
	 * \param aMeeting The meeting which is seeked in the list for.
	 * \return True if contains; otherwise, false.
	 */
	static bool isMeetingInList( const QList<Meeting*> &aList, const Meeting *aMeeting );
	//! Slot. Fetches meetings from the server.
	/*!
	 * Slot. Fetches meetings from the server, exact parameters are specified in the parameter list.
	 * \param aFrom Time from when the meetings need to be fetched.
	 * \param aUntil Time until when the meetings need to be fetched.
	 * \param aIn The room which meetings need to be fetched.
	 */
	void fetchMeetings( const QDateTime &aFrom, const QDateTime &aUntil, const Room *aIn );

private:
	static QTime endOfTheDay;

	WindowManager *iWindowManager;
	QTimer *iIdleTimeCounter;
	Clock *iClock;
	Configuration *iConfiguration;
	CommunicationManager *iCommunication;
	DeviceManager *iDevice;

	QTimer *iAutoRefresh;

	QList<Meeting*> iMeetings;
};

#endif /*ENGINE_H_*/
