#include "Communication.h"
#include "ConnectionSettings.h"
#include <QAuthenticator>

Communication::Communication( const ConnectionSettings &aConnection ) :
	iCurrentRequest(0),
	iAuthFailCount(0)
{
	iConnectionSettings = new ConnectionSettings( aConnection );

	iHttp = new QHttp( iConnectionSettings->serverUrl().toString(), QHttp::ConnectionModeHttps );

	connect( iHttp,
			 SIGNAL( readyRead( const QHttpResponseHeader& ) ),
			 this,
			 SLOT( processResponse( const QHttpResponseHeader& ) )
			);
	connect( iHttp,
			 SIGNAL( requestStarted( int ) ),
			 this,
			 SLOT( handleRequestStarted( int ) )
			);
	connect( iHttp,
			 SIGNAL( requestFinished( int, bool ) ),
			 this,
			 SLOT( handleResults( int, bool ) )
			);
	connect( iHttp,
			 SIGNAL( dataReadProgress( int, int ) ),
			 this,
			 SLOT( handleReadProgress( int, int ) )
			);
	connect( iHttp,
			 SIGNAL( authenticationRequired( const QString&, quint16, QAuthenticator* ) ),
			 this,
			 SLOT( handleAuthentication( const QString&, quint16, QAuthenticator* ) ) );
	connect( iHttp,
			 SIGNAL( sslErrors( const QList<QSslError>& ) ),
			 iHttp,
			 SLOT( ignoreSslErrors() )/*this, SLOT( notifySsl( const QList<QSslError>& ) )*/
			);
}

Communication::~Communication()
{
	delete iConnectionSettings;
	delete iHttp;
	QList<QByteArray*> responses = iResponses.values();
	while(!responses.isEmpty())
		delete responses.takeFirst();
}

void Communication::processResponse( const QHttpResponseHeader& aHeader )
{
	if ( aHeader.statusCode() == 200 )
	{
		if( iResponses.contains( iCurrentRequest ) )
		{
			QByteArray* response = iResponses.value( iCurrentRequest, NULL );
			response->append( iHttp->readAll() );
		}
	}
}

void Communication::handleRequestStarted( int aRequestId )
{
	if( iResponses.contains( aRequestId ) )
	{
		iCurrentRequest = aRequestId;
		emit requestStarted( aRequestId );	
   	}
}

void Communication::handleResults( int aId, bool /*aError*/ )
{
	if( iCurrentRequest != 0 && aId == iCurrentRequest )
	{
		iAuthFailCount = 0;
		iCurrentRequest = 0;
		int err = (int)iHttp->error();
		if( iHttp->error() == QHttp::Aborted && iAuthFailCount > 1 )
			err = 11;
		emit requestFinished( aId, err );
	}
}

void Communication::handleAuthentication( const QString& /*aHost*/, quint16 /*aPort*/, QAuthenticator* aAuthenticator )
{
	aAuthenticator->setPassword( iConnectionSettings->password() );
	aAuthenticator->setUser( iConnectionSettings->username() );
	iAuthFailCount++;
	if( iAuthFailCount > 1 )
	{
		iHttp->abort();
	}
}

int Communication::request( const QString &aCommand, const QByteArray &aContent )
{
	if( iAuthFailCount > 1 )
		return 0;
	
	QHttpRequestHeader header( QString( "POST" ), QString( "/ews/exchange.asmx" ) );
	header.setValue( "Host", iConnectionSettings->serverUrl().toString() );
	QString command = aCommand;
	header.setContentType( command );

	int id = iHttp->request( header, aContent );
	if( iResponses.contains( id ) )
	{
		QByteArray* response = iResponses.value( id, NULL );
		response->clear();
	}
	else
	{
		iResponses.insert( id, new QByteArray() );
	}
	
	return id;
}

QByteArray* Communication::response( int aRequestId )
{
	if( !iResponses.contains( aRequestId ) )
		return NULL;
	QByteArray* response = iResponses.take( aRequestId );
	return response;
}

void Communication::handleReadProgress( int aDone, int aTotal )
{
	emit readProgress( iCurrentRequest, aDone, aTotal );
}
