#ifndef COMMUNICATIONMANAGER_H_
#define COMMUNICATIONMANAGER_H_

#include <QObject>
#include <QList>
#include <QDateTime>
#include "Communication.h"
#include "Room.h"
#include "MessagingUtils.h"

class ConnectionSettings;
class Meeting;

//! Responsible for Exchange server communication and request encoding and response parsing.
/*!
 *  CommunicationManager holds two connections to the Exchange server:
 *  Fetching and modifying connection. Fetching connetion is read only connection
 *  which has static authentication information defined in the application configuration.
 *  The modifying connection's authentication information varies, according to the user interface.
 * 
 *  NOTE! Currently Modifying connection is NOT implemented.
 */
class CommunicationManager : public QObject
{
	Q_OBJECT

public:
	enum CommunicationType
	{
		FetchingCommunication,
		ModifyingCommunication
	};

public:
	//! Constructor.
	/*! 
	 *  \param aConnection Reference to the fetching ConnectionSettings.
	*/
	CommunicationManager( const ConnectionSettings &aConnection );
	virtual ~CommunicationManager();
	//! Starts fetching meetings. Meetings are returned by the meetingsFetched signal.
	/*! 
	 *  Calls the MS Exchange GetUserAvailability WebService method. 
	 *  \param aFrom QDateTime to start searching from.
	 *  \param aUntil QDateTime to search meetings until.
	 *  \param aIn Meeting room meetings are searched.
	*/
	void fetchMeetings( const QDateTime &aFrom, const QDateTime &aUntil, const Room &aIn );
	//! Starts fetching a meeting details. Details are returned by the meetingDetailsFetched signal.
	/*! 
	 *  \param aMeeting A meeting the detailed information is wanted.
	*/
	void fetchMeetingDetails( Meeting &aMeeting );
	//! Cancels all meeting detail requests.
	void cancelFetchMeetingDetails();
/* Not supported member functions which are using the modifying communication
	void setModifyCredentials( const QString &aUsername, const QString &aPassword ) {};
	void createMeeting( const Meeting &aMeeting ) {};
	void updateMeeting( const Meeting &aMeeting ) {};
	void deleteMeeting( const Meeting &aMeeting ) {};
*/

signals:
	//! Emitted when an error happens. Error could be QHttp or SOAP related.
	/*! 
	*   \param aCode An error code defined by CommunicationManager.
	*   \param aType FetchingCommunication or ModifyingCommunication.
	*/
	void error( int aCode, CommunicationManager::CommunicationType aType );
	//! Emitted when a http request is ongoing. Reports the bytes read from the server.
	/*! 
	*   \param aDone Bytes read from the server.
	*   \param aTotal Bytes total of the response.
	*   \param aType FetchingCommunication or ModifyingCommunication.
	*/
	void readProgress( int aDone, int aTotal, CommunicationManager::CommunicationType aType );
	//! Emitted when meetings are fetched.
	/*! 
	*   \param aMeetings Meetings found according to the search criteria.
	*/
	void meetingsFetched( const QList<Meeting*> &aMeetings );
	//! Emitted when meeting details are fetched.
	/*! 
	*   \param aDetailedMeeting Meeting which contains detailed information.
	*/
	void meetingDetailsFetched( Meeting &aDetailedMeeting );

protected slots:
	//! Connected to Communication::requestFinished.
	void requestFinished( int aRequestId, int aError );
	//! Connected to Communication::readProgress.
	void readProgress( int aRequestId, int aDone, int aTotal );
	
protected:
	//! Gets the secondary id number of a meeting.
	/*! 
	 *  Calls the MS Exchange ConvertId WebService method. Converts primary id
	 *  number to secondary id number.
	 *  \param aMeeting A meeting which contains primaryId number.
	*/
	void getSecondaryIdForMeeting( Meeting &aMeeting );
	//! Gets the details of a meeting.
	/*! 
	 *  Calls the MS Exchange GetCalendarItem WebService method.
	 *  \param aMeeting A meeting the detailed information is wanted.
	*/
	void getMeetingDetails( Meeting &aMeeting );
	
	/*! 
	 *  Lists the WebService methods known by the CommunicationManager.
	*/
	enum RequestType
	{
		GetUserAvailability = 1,
		ConvertId,
		GetCalendarItem
	};
	//! Internal data structure which contains data about requests made to Exchange server.
	class RequestData
	{
	public:
		RequestData( RequestType aType, int aRequestId ) : 
			type( aType ), requestId( aRequestId ), room( NULL ), meeting( NULL ) {};
		~RequestData()
		{
			if( type == GetUserAvailability )
				delete room;
		};
		//! Holds one of the enum RequestType value
		RequestType type;
		//! Request ID number returned by the Communication::request
		int requestId;
		//! Pointer to a Room related to this request. NULL by default. Owned.
		Room* room;
		//! Pointer to a Meeting related to this request. Not owned.
		Meeting* meeting;
	};
	//! Adds a request to the iRequestInfos list.
	/*! 
	 *  \param aType One of the enum RequestType value.
	 *  \param aRequestId Request ID number returned by the Communication::request
	*/
	void addRequest( RequestType aType, int aRequestId );
	//! Takes RequestData off from the iRequestInfos list.
	/*! 
	 *  Returns a pointer to RequestData data structure. Removes RequestData from
	 *  the iRequestInfos list.
	 *  \param aRequestId Request ID number returned by the Communication::request
	*/
	RequestData* takeRequest( int aRequestId );
	//! Finds a RequestData from the iRequestInfos list.
	/*! 
	 *  Returns a pointer to RequestData, NULL if not found.
	 *  \param aRequestType Request type
	*/
	const RequestData* findRequest( RequestType aRequestType ) const;
	
private:
	//! Instance of Connection settings of the fetching communication
	ConnectionSettings *iConnectionSettings;
	//! Instance of the fetching communication
	Communication* iFetchingCommunication;
	//! Instance of the modifying communication
	Communication* iModifyingCommunication;
	//! Temporary list which holds the lastest meetings fetched
	QList<Meeting*> iMeetings;
	//! Additional information about requests made to the Exchange server
	QList<RequestData*> iRequestInfos;
	//! A flag that all meeting detail requests has been cancelled
	QList<int> iCancelledRequests;
};

#endif /*COMMUNICATIONMANAGER_H_*/
