#include "WindowManager.h"

#include <QTimer>
#include "Configuration.h"
#include "DisplaySettings.h"
#include "Meeting.h"
#include "Room.h"
#include "Clock.h"
#include "WeeklyViewWidget.h"
#include "RoomStatusIndicatorWidget.h"
#include "MeetingInfoDialog.h"
#include "PopUpMessageBox.h"
#include "DeviceManager.h"
#include "SettingsView.h"
#include "ProgressBar.h"

#include <QtDebug>

WindowManager::WindowManager( Configuration *aConfiguration ) :
		QObject(),
		iApplicationName( tr( "Qt Meetings" ) ),
		iFullScreen( false ),
		iConfiguration( aConfiguration ),
		iWeeklyView( 0 ),
		iRoomStatusView( 0 ),
		iMeetingInfo( 0 ),
		iProgressBar( 0 ),
		iPasswordDialog( 0 )
{
	iWeeklyView = new WeeklyViewWidget( QDateTime::currentDateTime(), aConfiguration );
	iWeeklyView->setWindowTitle( iApplicationName );
	connect( iWeeklyView, SIGNAL( observedEventDetected() ), this, SIGNAL( observedEventDetected() ) );
	connect( iWeeklyView, SIGNAL( meetingActivated( Meeting * ) ), this, SIGNAL( meetingActivated( Meeting * ) ) );
	connect( iWeeklyView, SIGNAL( currentRoomChanged( Room * ) ), this, SIGNAL( currentRoomChanged( Room * ) ) );
	connect( iWeeklyView, SIGNAL( shownWeekChanged( QDate ) ), this, SIGNAL( shownWeekChanged( QDate ) ) );
	
	showWeeklyView();
	
}

WindowManager::~WindowManager()
{
	delete iWeeklyView;
	iWeeklyView = 0;
	delete iRoomStatusView;
	iRoomStatusView = 0;
	delete iMeetingInfo;
	iMeetingInfo = 0;
	delete iProgressBar;
	iProgressBar = 0;
	delete iPasswordDialog;
	iPasswordDialog = 0;
}

void WindowManager::distributeDateTimeInfo( QDateTime aCurrentDateTime )
{
	if ( iRoomStatusView != 0 && iRoomStatusView->isActiveWindow() )
	{
		iRoomStatusView->setCurrentTime( aCurrentDateTime.time() );
	}

	if ( iWeeklyView != 0 && iWeeklyView->isActiveWindow() )
	{
		iWeeklyView->setCurrentDateTime( aCurrentDateTime );
	}
}

void WindowManager::roomStatusChanged( Room *aRoom, Room::Status aStatus, QTime aTime )
{
	if ( iRoomStatusView == 0 )
	{
		iRoomStatusView = new RoomStatusIndicatorWidget( aRoom, aStatus, aTime, iConfiguration->displaySettings()->timeFormat() );
		iRoomStatusView->setWindowTitle( iApplicationName );
		if( iFullScreen )
			iRoomStatusView->setWindowState( Qt::WindowFullScreen );
		connect( iRoomStatusView, SIGNAL( observedEventDetected() ), this, SIGNAL( observedEventDetected() ) );
	}
	else
	{
		iRoomStatusView->statusChanged( aStatus, aTime );
	}
	if ( !iWeeklyView->isVisible() && !iRoomStatusView->isVisible() )
	{
		showRoomStatus();
	}
}

void WindowManager::showRoomStatus()
{
	qDebug() << "WindowManager::showRoomStatus";

	if ( iRoomStatusView == 0 )
	{
		emit roomStatusInfoNeeded( iWeeklyView->currentRoom() );
	}
	else
	{
		iRoomStatusView->show();
		if ( iWeeklyView->isVisible() )
		{
			iWeeklyView->hide();
		}
	}

	// closing/deleting meeting info dialog
	if ( iMeetingInfo != 0 )
	{
		iMeetingInfo->hide();
	}
}

void WindowManager::showWeeklyView()
{
	qDebug() << "WindowManager::showWeeklyView";
	if ( iRoomStatusView != 0 && iRoomStatusView->isVisible() )
	{
		iRoomStatusView->hide();
	}

	iWeeklyView->show();
}

void WindowManager::fullScreen()
{
	if ( iRoomStatusView != 0 )
		iRoomStatusView->setWindowState( Qt::WindowFullScreen );
	if ( iWeeklyView != 0 )
		iWeeklyView->setWindowState( Qt::WindowFullScreen );
	iFullScreen = true;
}

void WindowManager::insertMeeting( Meeting *aMeeting )
{
	iWeeklyView->insertMeeting( aMeeting );
}

void WindowManager::deleteMeeting( Meeting *aMeeting )
{
	iWeeklyView->deleteMeeting( aMeeting );
}

void WindowManager::showMeetingInfo( Meeting *aMeeting )
{
	iMeetingInfo = new MeetingInfoDialog( aMeeting );
	// Display modal dialog
	iMeetingInfo->exec();

	delete iMeetingInfo;
	iMeetingInfo = 0;
}

void WindowManager::showSettingsView()
{
	// TODO : give the Torspo for the person who was responsible to write this method
}

WeeklyViewWidget * WindowManager::weeklyView()
{
	return iWeeklyView;
}

void WindowManager::error( const QString &aErrorMessage )
{
	qDebug() << "WindowManager::showErrorPopup";

	PopUpMessageBox::error( 0, aErrorMessage );
}

void WindowManager::showPasswordDialog( QByteArray aAdminPassword, const QString &aMessage )
{
	iPasswordDialog = new PasswordDialog( aAdminPassword, aMessage );
	connect( iPasswordDialog, SIGNAL( passwordEntered( PasswordDialog::PasswordStatus ) ),
			this, SIGNAL( passwordEntered( PasswordDialog::PasswordStatus ) ) );
	iPasswordDialog->show();
	
	//TODO connect connect( iWeeklyView, SIGNAL( observedEventDetected() ), this, SIGNAL( observedEventDetected() ) );
}

void WindowManager::closePasswordDialog()
{
	iPasswordDialog->close();
	delete iPasswordDialog;
	iPasswordDialog = 0;
}

void WindowManager::showProgressBar( const QString &aText, bool aCancellable )
{
	qDebug() << "WindowManager::showProgressBar( const QString & )";
	if( iProgressBar == 0 ) {
		iProgressBar = new ProgressBar( aText, aCancellable );
		iProgressBar->setFixedSize( 600, 125 );
		iProgressBar->show();
		if( aCancellable )
			connect( iProgressBar, SIGNAL( cancel() ), this, SIGNAL( progressBarCancelled() ) );
	}
	
	//TODO connect connect( iWeeklyView, SIGNAL( observedEventDetected() ), this, SIGNAL( observedEventDetected() ) );
}

void WindowManager::closeProgressBar()
{
	qDebug() << "WindowManager::closeProgressBar()";
	if( iProgressBar )
	{
		iProgressBar->close();
		delete iProgressBar;
		iProgressBar = 0;
	}
}

void WindowManager::updateProgressBar( const QString &aMessage )
{
	qDebug() << "WindowManager::updateProgressBar( const QString & )";
	if( iProgressBar != 0 )
		iProgressBar->update( aMessage );
}
