#include <QtTest/QtTest>
#include "Configuration.h"
#include "ConnectionSettings.h"
#include "TestCommunicationManager.h"
#include "Communication.h"
#include "Room.h"
#include "Meeting.h"
#include <QString>
#include <QSignalSpy>
#include <QDateTime>

static const QString server = "192.168.0.35";
static const QString user   = "maemo";
static const QString pass   = "P@ssw0rd";
static const unsigned int refresh = 1;

void TestCommunicationManager::initTestCase()
{
    ConnectionSettings *conn = new ConnectionSettings( server, user, pass, refresh );
    QVERIFY( conn != 0 );
	iMgr = new CommunicationManager( *conn );
	QVERIFY( iMgr != 0 );
    delete conn;
    conn = NULL;
}

void TestCommunicationManager::cleanupTestCase()
{
	delete iMgr;
}

void TestCommunicationManager::init() { }

void TestCommunicationManager::cleanup() { }

void TestCommunicationManager::testErrorSignal()
{
	ConnectionSettings *conn = new ConnectionSettings( QString(""), user, pass, refresh );
	CommunicationManager *mgr = new CommunicationManager( *conn );
	delete conn;

	QDateTime start = QDateTime::currentDateTime();
	QDateTime end = start.addDays(14);
	Room room( "Room name","Room address" );

	connect( mgr,
			 SIGNAL( error( int, CommunicationManager::CommunicationType  ) ),
			 this,
			 SLOT( handleError(int, CommunicationManager::CommunicationType  ) )
			);
	
	iError = 0;
	mgr->fetchMeetings( start, end, room );
	for(int i=0; iError == 0 && i < 5000/250; i++ )
		QTest::qWait( 250 );
	
	if( iError == 0 )
	{
		QFAIL( "error signal not emitted" );
	}
	
	delete mgr;
}

void TestCommunicationManager::testFetchMeetings()
{
	qRegisterMetaType<CommunicationManager::CommunicationType>("CommunicationManager::CommunicationType");
	QSignalSpy rp( iMgr, SIGNAL(readProgress( int, int, CommunicationManager::CommunicationType )) );
	QCOMPARE( rp.isValid(), true );

	connect( iMgr,
			 SIGNAL( meetingsFetched( const QList<Meeting*>& ) ),
			 this,
			 SLOT( meetingsFetched( const QList<Meeting*>& ) )
			);
	connect( iMgr,
			 SIGNAL( error( int, CommunicationManager::CommunicationType  ) ),
			 this,
			 SLOT( handleError(int, CommunicationManager::CommunicationType  ) )
			);
	QDateTime start = QDateTime::fromString( QString("04.05.2009"), QString("dd.MM.yyyy") );
	QDateTime end = QDateTime::fromString( QString("09.05.2009"), QString("dd.MM.yyyy") );
	Room room( "TestRoom", "meetingroomtest@test.local" );
	iMgr->fetchMeetings( start, end, room );
	iError = 0;
	for(int i=0; iError == 0 && i < 5000/250; i++ )
		QTest::qWait( 250 );

	if( iError == 0 )
	{
		QFAIL("signal meetingsFetched not emitted");
	}

	QVERIFY( iMeetings.count() > 0 );
	for( int i=0; i < iMeetings.count(); i++)
	{
		Meeting *m = iMeetings[i];
		qDebug() << m->toString();
	}

	//verify readProgress
	QCOMPARE( rp.count(), 1 );
	QList<QVariant> args = rp.takeFirst();
	QCOMPARE( args.at(0).type(), QVariant::Int );
	QCOMPARE( args.at(1).type(), QVariant::Int );
	QCOMPARE( args.at(2).typeName(), "CommunicationManager::CommunicationType" );
}

void TestCommunicationManager::testFetchMeetingDetails()
{
	connect( iMgr,
			 SIGNAL( meetingDetailsFetched( Meeting& ) ),
			 this,
			 SLOT( meetingDetailsFetched( Meeting& ) )
			);
	Meeting* m = iMeetings[0];
	QVERIFY( m->secondaryId().isEmpty() );
//	QVERIFY( !m->primaryId().isEmpty() );
	iMgr->fetchMeetingDetails( *m );
	iError = 0;
	for(int i=0; iError == 0 && i < 5000/250; i++ )
		QTest::qWait( 250 );

	if( iError == 0 )
	{
		QFAIL("signal meetingDetailsFetched not emitted");
	}
	QVERIFY( !m->secondaryId().isEmpty() );
	QVERIFY( m->detailsAvailable() );
	qDebug() << m->toString();
}

void TestCommunicationManager::handleError( int aCode, CommunicationManager::CommunicationType aType )
{
	QWARN("TestCommunicationManager::handleError");
	QCOMPARE( aType, CommunicationManager::FetchingCommunication );
	QVERIFY( aCode != 0 );
	iError = aCode;
}

void TestCommunicationManager::meetingsFetched( const QList<Meeting*> &aMeetings )
{
	iMeetings = aMeetings;
	iError = 1;
}

void TestCommunicationManager::meetingDetailsFetched( Meeting &aDetailedMeeting )
{
	QCOMPARE( &aDetailedMeeting, iMeetings[0] );
	iError = 1;
}
